#!/usr/bin/perl -w
# Mark Kamichoff <mark.kamichoff@twcable.com>
# ./snmptraf.pl - Graphs interface stats via SNMPv1/v2c given host,
# community string, and interface name.
# All rates and peaks are in Mb/sec (MegaBits/sec)
# Version 0.1
#
# Copyright (c) 2007, Mark Kamichoff
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the <organization> nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Mark Kamichoff ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

use Term::ReadKey;
use Term::ANSIColor; 
use Getopt::Std;
use Net::SNMP;
use Time::HiRes qw(usleep tv_interval);
use POSIX qw(ceil floor);

# No dynamic resizing, yet
$maxstars = 21;

# Standard OIDs
$ifDescrs = '1.3.6.1.2.1.2.2.1.2';
$ifSpeeds = '1.3.6.1.2.1.2.2.1.5';

sub syntax {
#	print "Syntax: $ENV{'_'} <-t time in us> <-v 1|2c> -c [community] -h [host] -i [interface]\n";
	print "Syntax: <-t time in us> <-v 1|2c> -c [community] -h [host] -i [interface]\n";
	exit 1;
}

%options=();
if(!getopts("t:v:c:h:i:", \%options)) {
	syntax();
}

# check optionals
if(defined $options{'t'}) {
	$interval = $options{'t'};
} else {
	$interval = 150000;	# default at 150ms
}
if(defined $options{'v'}) {
	$version = $options{'v'};
} else {
	$version = "1";
}

# check required arguments
if(defined($options{'c'}) && defined($options{'h'}) && defined($options{'i'})) {
	$community = $options{'c'};
	$host = $options{'h'};
	$ifName = $options{'i'};
} else {
	syntax();
}

# init
($session,$error) = Net::SNMP->session(-hostname => $host,
		-community => $community,
		-version => $version);

die "SNMP session error: $error" unless ($session);

# get interface names and speeds
$ifDescrsRef = $session->get_table($ifDescrs) ||
		die "Error walking ifDescr: $error" unless ($ifDescrsRef);
$ifSpeedsRef = $session->get_table($ifSpeeds) ||
		die "Error walking ifSpeed: $error" unless ($ifSpeedsRef);

# search for specified name, to get idx
foreach $key (keys %{$ifDescrsRef}) {
	if($ifName eq $ifDescrsRef->{$key}) {
		$key =~ m/\.(\d+)$/;
		$idx = $1;
		last;
	}
}

# invalid interface name, display candidates
if(!defined($idx)) {
	print "Error: Interface name $ifName is invalid.\n";
	print "The following interfaces were found on $host:\n";

	# show all interfaces, and speeds
	foreach $key (keys %{$ifDescrsRef}) {
		$key =~ m/\.(\d+)$/;
format List =
	* [@|] @<<<<<<<<<<<<<<<<< @<<<<<<<<<<<<<<<<<<<<<
$fmtIdx, $fmtName, $fmtSpeed,
.
		$fmtIdx = $1;
		$fmtName = $ifDescrsRef->{$key};
		$fmtSpeed = sprintf("%.2f Mb/sec",
				$ifSpeedsRef->{$ifSpeeds.'.'.$1}/1000000);
		$~ = 'List';
		write;
	}

	exit(1);
}

# in/out
@peaks = ( 0, 0 );

# Loop through, until ^C
for($count = 0; ; $count++ ) {

	# Print the column lables, every $LINES time
	@termdim=GetTerminalSize();
	if(!($count % $termdim[1]) || !$count) {
		print color 'bold cyan';
		print "Peak Inbound                     Rate | Rate                     Peak Outbound\n";
		print color 'reset';
		print "------------------------------------------------------------------------------\n";
	}

	# In
	$t0 = [Time::HiRes::gettimeofday];
	$result[0] = $session->get_request(".1.3.6.1.2.1.2.2.1.10.$idx");
	die "SNMP request error: ".$session->error unless (defined $result[0]);
	$lag[0] = tv_interval ($t0, [Time::HiRes::gettimeofday]);
	$cur[0] = $result[0]->{".1.3.6.1.2.1.2.2.1.10.$idx"};

	# Out
	$t0 = [Time::HiRes::gettimeofday];
	$result[1] = $session->get_request(".1.3.6.1.2.1.2.2.1.16.$idx");
	die "SNMP request error: ".$session->error unless (defined $result[1]);
	$lag[1] = tv_interval ($t0, [Time::HiRes::gettimeofday]);
	$cur[1] = $result[1]->{".1.3.6.1.2.1.2.2.1.16.$idx"};

	if(@old) {
		foreach $i ( 0 .. 1 ) {
			# Mb/sec
			$rates[$i] = ( ($cur[$i] - $old[$i]) / 
					( ($interval/1000000) + $lag[$i] ))
					/ 125000;
			$peaks[$i] = $rates[$i] unless ($rates[$i] <= $peaks[$i]);
		}

# the bar graphs are $maxstars ch/ea
format Graph =
@<<<<<< @>>>>>>>>>>>>>>>>>>>> @>>>>>> @|@<<<<< @<<<<<<<<<<<<<<<<<<<<<< @>>>>>>
$t_peak[0], $t_graph[0], $t_rate[0], $pipe, $t_rate[1], $t_graph[1], $t_peak[1],
.

		$t_peak[0] = sprintf("%.1f", $peaks[0]);
		$t_peak[1] = sprintf("%.1f", $peaks[1]);
		$t_rate[0] = sprintf("%.1f", $rates[0]);
		$t_rate[1] = sprintf("%.1f", $rates[1]);
		$pipe = "|";

		# Draw the graph (0=in, 1=out)
		for $j ( 0 .. 1 ) {
			$t_graph[$j] = "";
			if($peaks[$j]) {
				$len = floor($maxstars*$rates[$j])/$peaks[$j];
				for($i=0; $i < $len; $i++) {
					$t_graph[$j] = $t_graph[$j]."*";
				}
			}
		}
		$~ = 'Graph';
		write;
	}

	@old = @cur;

	usleep($interval);
}

# We won't get here
$session->close;
